@@ -3,7 +3,7 @@ use warnings;
 
 package Git::More;
 {
-  $Git::More::VERSION = '0.015';
+  $Git::More::VERSION = '0.016';
 }
 # ABSTRACT: An extension of App::gh::Git with some goodies for hook developers.
 use parent 'App::gh::Git';
@@ -80,6 +80,19 @@ sub get_commit_msg {
 }
 
 
+sub get_affected_files {
+    my ($git, $old_commit, $new_commit, $filter) = @_;
+    my @cmd = qw/diff --cached --name-status/;
+    push @cmd, "--diff-filter=$filter" if $filter;
+    my %affected;
+    foreach ($git->command(@cmd)) {
+	my ($status, $name) = split / /, $_, 2;
+	$affected{$name} = $status;
+    }
+    return \%affected;
+}
+
+
 1;
 
 __END__
@@ -91,7 +104,7 @@ Git::More - An extension of App::gh::Git with some goodies for hook developers.
 
 =head1 VERSION
 
-version 0.015
+version 0.016
 
 =head1 SYNOPSIS
 
@@ -194,6 +207,17 @@ codes are explained in the C<git help rev-list> document):
 This method returns the commit message (aka body) of the commit
 identified by COMMIT_ID. The result is a string.
 
+=head2 get_affected_files OLDCOMMIT NEWCOMMIT [FILTER]
+
+This method returns a reference to a hash mapping every affected files
+between OLDCOMMIT and NEWCOMMIT to their affecting status. The list is
+grokked with the command C<git diff --cached --name-status>.
+
+The optional FILTER parameter must be a valid value for the
+C<--diff-filter> option of the C<git diff> command. You can use it to
+C<AM>, for instance, to request only files that have been Added or
+Modified.
+
 =head1 SEE ALSO
 
 C<App::gh::Git>
